#!/usr/bin/perl

# Perl OSM Logger

# v0.1 20070801: Initial run
# v0.2 20070802: Added Readline history, tab completion
# v0.21 20070802: Use Time::HiRes for 0.1 second sleep
# v0.3 20070810: Use HTML::FromText to encode characters as HTML entities as required

use Net::GPSD;
use Term::ReadKey;
use Term::ReadLine;
use Time::HiRes qw( usleep ualarm gettimeofday tv_interval );
use HTML::FromText;

my $gpsd=Net::GPSD->new(host=>"localhost", port=>"2947");

my $point, @list, $list, $rate, $identification, $protocol, $daemon;
my $satcount, $satsnr, $satused = 0;

my $done, $key, $line, $encodedline;

my $sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst;

my $lat, $lon, $ele, $time;

my $i;

my $rlname = "posmlog";
my $term = Term::ReadLine->new($rlname);
my $attribs = $term->Attribs;

my $defaultcompletionfile = "posmlog.completion";
my @completionwords;

open FH, "<$defaultcompletionfile";
$i = 0;
while (<FH>) {
  chomp ($_);
  $completionwords[$i] = $_;
  print "Completion word $i is ".$completionwords[$i]." from ".$_."\n";
  $i++;
}
$attribs->{completion_word} = \@completionwords;
$attribs->{completion_append_character} = ' ';
$attribs->{completion_entry_function} = $attribs->{'list_completion_function'};

# Create a filename based off the current date & time (down to seconds)
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
$year = $year + 1900;
$mon++;
if ($mon < 10) {
  $mon = "0".$mon;
}
if ($mday < 10) {
  $mday = "0".$mday;
}
if ($sec < 10) {
  $sec = "0".$sec;
}
my $logfile = $year.$mon.$mday.$hour.$min.$sec;

open LOG, ">>$logfile.gpx";

# Prep the XML header
print LOG "<?xml version=\"1.0\" encoding=\"utf-8\"?>
<gpx version=\"1.1\" creator=\"$rlname\"
        xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"
        xmlns=\"http://www.topografix.com/GPX/1.1\"
        xsi:schemaLocation=\"http://www.topografix.com/GPS/1/1
        http://www.topografix.com/GPX/1/1/gpx.xsd\">
 <metadata>
  <name>posmlog </name>
  <author>Steve Bromwich</author>
  <copyright>GPL v 2.0</copyright>
 </metadata>\n";

# Check for input
sub process_input {
  my $key = shift;
  $done = $key eq 'q';
  if ($key eq 'm') {
    # Reset mode to get echo
    ReadMode ('normal');
    print "\n";
    $line = $term->readline('marker>');
    if ($line) {
      # convert characters to HTML entities as required
      $encodedline = text2html ($line);
      if ($encodedline == $line) {
        print "\nMarker: $encodedline\n";
      } else {
        print "\nMarker encoded: $encodedline\n";
      }
      print LOG "    <wpt lat=\"$lat\" lon=\"$lon\">
    <name>$encodedline</name>
   </wpt>\n";
    }
  }
  ReadMode ('normal');
  if ($done) {
    print LOG "</gpx>\n";
    close LOG;
    exit;
  }
}



# Main loop
while (1) {
  my $p = $gpsd->get();
  if ($p->fix) {
    print join("|", qw{mode time lat lon alt speed heading}), "\n";
    print join("|",  map {defined $_?$_:''}
      $p->mode,
      $p->time,
      $p->lat,
      $p->lon,
      $p->alt,
      $p->speed,
      $p->heading),
    "\n";
    $lat = $p->lat;
    $lon = $p->lon;
    $ele = $p->alt;
    $time = $p->time;
  } else {
    print "No fix\n";
  }
  @list = $gpsd->getsatellitelist;
  $list = $gpsd->getsatellitelist;

  $rate = $gpsd->rate;
  $id = $gpsd->identification;
  $protocol = $gpsd->protocol;

  $daemon = $gpsd->daemon;
  
  $satcount = $satsnr = $satused = 0;
  foreach ($gpsd->getsatellitelist) {
    $satcount++;
    $satsnr = $satsnr + $_->snr;
    if ($_=>used eq "1") {
      $satused++;
    }
  }

  # Check for div by zero
  if ($satcount == 0) {
    $satsnr = 0;
    $satused = 0;
  } else {
    if ($satsnr > 0  && $satused > 0) {
      $satsnr = $satsnr / $satused;
    } elsif ($satsnr > 0 && $satused == 0) {
      # If we don't have a lock base SNR off total count of satellites seen
      $satsnr = $satsnr / $satcount;
    }
  }
  
  print "$satused sats used, ave SNR $satsnr, $satcount total sats seen, id $id, proto $protocol\n";

  for ($i = 0; $i < 10 ; $i++) {
    # Cheap hack, sleep for .1 seconds to make sure we're not busywaiting and burning battery
    usleep 100000;
    # Check for input
    ReadMode( 'cbreak' );
    if (defined ($key = ReadKey (-1))) {
      process_input ($key);
      $i = 10;
    }
  }
}

